<?php

namespace Drupal\layout_builder_ipe\Access;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\layout_builder_ipe\LayoutBuilderIpeService;
use Symfony\Component\DependencyInjection\ContainerInterface;

class LayoutBuilderPageAccess extends ControllerBase {

  /**
   * The route match.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * The layout builder ipe service.
   *
   * @var \Drupal\layout_builder_ipe\LayoutBuilderIpeService
   */
  protected $layoutBuilderIpe;

  /**
   * Constructs a new RouteCacheContext class.
   *
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The route match.
   */
  public function __construct(RouteMatchInterface $route_match, LayoutBuilderIpeService $layout_builder_ipe) {
    $this->routeMatch = $route_match;
    $this->layoutBuilderIpe = $layout_builder_ipe;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('current_route_match'),
      $container->get('layout_builder_ipe')
    );
  }

  /**
   * Access callback for the "Layout" page on nodes.
   *
   * @return void
   */
  public function accessLayoutPage() {
    // See if the access to the layout page should be disabled.
    if (!$this->layoutBuilderIpe->getConfig('disable_layout_page')) {
      // Returning allows to let the normal layout builder access logic take
      // effect.
      return AccessResult::allowed();
    }
    /** @var \Drupal\node\NodeInterface $node */
    $node = $this->routeMatch->getParameter('node');
    if ($node) {
      $bundle = $node->bundle();
      $entity_type_id = $node->getEntityTypeId();
      $permission = "use layout builder ipe on editable $bundle $entity_type_id layout overrides";
      if (!$this->currentUser()->hasPermission($permission)) {
        // No permission to use IPE, so the layout page should be available.
        return AccessResult::allowed();
      }

      // Now we trigger a view build, just so that we can see which view mode
      // (with which settings) would most probably be used if this was a full
      // page rendering. This doesn't seem neither elegent nor very performant,
      // but I didn't see another way of getting the view mode.
      // Luckily the result of this access check is cached.
      /** @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface $entity_display_repository */
      $entity_display_repository = \Drupal::getContainer()->get('entity_display.repository');
      $view_builder = \Drupal::entityTypeManager()->getViewBuilder($node->getEntityTypeId());
      $build = $view_builder->view($node);
      $view_mode = $build['#view_mode'];

      // If the full view mode is not enabled and can therefor not be
      // configured separatly, we have to look at the default view mode which
      // is used instead.
      $available_view_modes = $entity_display_repository->getViewModeOptionsByBundle($entity_type_id, $bundle);
      if ($view_mode == 'full' && !array_key_exists('full', $available_view_modes)) {
        $display = $entity_display_repository->getViewDisplay($entity_type_id, $bundle, 'default');
      }
      else {
        $display = $entity_display_repository->getViewDisplay($entity_type_id, $bundle, $view_mode);
      }

      if (!$display->getThirdPartySetting('layout_builder_ipe', 'enabled', FALSE)) {
        // If not enabled for this display, we allow access to the layout page.
        return AccessResult::allowed();
      }
    }

    // Prevent access.
    return AccessResult::forbidden();
  }

}