<?php

namespace Drupal\layout_builder_ipe;

use Drupal\Component\Plugin\Exception\ContextException;
use Drupal\Component\Utility\Html;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Theme\ThemeManagerInterface;
use Drupal\Core\Url;
use Drupal\layout_builder\LayoutEntityHelperTrait;
use Drupal\layout_builder\LayoutTempstoreRepositoryInterface;
use Drupal\layout_builder\SectionStorageInterface;
use Drupal\page_manager\Entity\PageVariant;
use Symfony\Component\HttpFoundation\RequestStack;

class LayoutBuilderIpeService {

  use LayoutEntityHelperTrait {
    LayoutEntityHelperTrait::getSectionStorageForEntity as layoutEntityHelperGetSectionStorageForEntity;
  }

  /**
   * The current route match.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * The current request.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The theme manager.
   *
   * @var \Drupal\Core\Theme\ThemeManagerInterface
   */
  protected $themeManager;

    /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The layout builder ipe config object.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $config;

  /**
   * Layout tempstore repository.
   *
   * @var \Drupal\layout_builder\LayoutTempstoreRepositoryInterface
   */
  protected $layoutTempstoreRepository;

  /**
   * Public constructor.
   */
  public function __construct(RouteMatchInterface $route_match, RequestStack $request_stack, EntityTypeManagerInterface $entity_type_manager, AccountInterface $current_user, ThemeManagerInterface $theme_manager, ModuleHandlerInterface $module_handler, ConfigFactoryInterface $config_factory, LayoutTempstoreRepositoryInterface $layout_tempstore_repository) {
    $this->routeMatch = $route_match;
    $this->request = $request_stack->getCurrentRequest();
    $this->entityTypeManager = $entity_type_manager;
    $this->currentUser = $current_user;
    $this->themeManager = $theme_manager;
    $this->moduleHandler = $module_handler;
    $this->layoutTempstoreRepository = $layout_tempstore_repository;
    $this->config = $config_factory->get('layout_builder_ipe.settings');
  }

  /**
   * Check access to the Layout Builder IPE frontend.
   *
   * @param \Drupal\layout_builder\SectionStorageInterface $section_storage
   *   The section storage.
   *
   * @return \Drupal\Core\Access\AccessResult
   *   An access result object.
   */
  public function access(SectionStorageInterface $section_storage, EntityInterface $entity = NULL) {
    if (!$section_storage->access('view')) {
      return AccessResult::forbidden();
    }
    if ($section_storage->getPluginId() == 'page_manager' && $this->currentUser->hasPermission('administer pages')) {
      return AccessResult::allowedIfHasPermission($this->currentUser, 'use layout builder ipe on editable page manager pages');
    }
    if ($entity && !$this->isLayoutCompatibleEntity($entity)) {
      return AccessResult::forbidden();
    }
    $entity = $this->getEntityFromSectionStorage($section_storage) ?? $entity;
    if (!$entity->access('update')) {
      return AccessResult::forbidden();
    }
    return AccessResult::allowedIfHasPermission($this->currentUser, "use layout builder ipe on editable {$entity->bundle()} {$entity->getEntityTypeId()} layout overrides");
  }

  /**
   * Get the current path for an edited page.
   *
   * Only relevant for page manager pages.
   *
   * @return string|null
   *   The current path if found.
   */
  public function getCurrentEditPath() {
    $query = $this->request->query;
    // The current path from the query was the original idea. Unfortunately,
    // the contextual links in layout builder (for configure, remove, etc ...)
    // do not respect existing query arguments, not even when modified via an
    // outbound url processor. Fortunately though, they use the destination
    // argument consistently, so it should be safe to use that.
    if ($query->has('current_path')) {
      return $query->get('current_path');
    }
    if ($query->has('destination')) {
      return $query->get('destination');
    }
  }

  /**
   * Attach the IPE frontend to the given build array
   *
   * @param array $build
   *   The render array to attach Layout Builder IPE to.
   * @param \Drupal\layout_builder\SectionStorageInterface $section_storage
   *   The section storage.
   */
  public function attachIpe(&$build, SectionStorageInterface $section_storage) {
    $entity = $this->getEntityFromSectionStorage($section_storage);
    if (!$this->access($section_storage, $entity)->isAllowed()) {
      return;
    }

    // Settings.
    $hide_local_tasks = $this->getConfig('hide_local_tasks_block');
    $enhance_ui = $this->getConfig('enhance_ui');

    $content_entity_plugins = ['overrides', 'defaults'];
    if ($entity instanceof ContentEntityInterface && in_array($section_storage->getPluginId(), $content_entity_plugins)) {
      $entity_target_class = Html::getClass('layout-builder-ipe-target');
      $build['#attributes']['class'][] = $entity_target_class;
      $entity_selector = '.' . $entity_target_class;

      $ipe_options = [
        'entity_selector' => $entity_selector,
        'hide_local_tasks' => $hide_local_tasks,
        'enhance_ui' => $enhance_ui
      ];

      if ($section_storage->getPluginId() == 'overrides') {
        $build['#attached']['drupalSettings']['layout_builder_ipe'] = [
          'urls' => [
            'customize' => Url::fromRoute('layout_builder_ipe.entity.edit', [
              'section_storage_type' => $section_storage->getStorageType(),
              'section_storage' => $section_storage->getStorageId(),
            ])->toString(),
          ],
        ] + $ipe_options;
      }
      if ($section_storage->getPluginId() == 'defaults') {
        $build['#attached']['drupalSettings']['layout_builder_ipe'] = [
          'urls' => [
            'customize' => Url::fromRoute('layout_builder_ipe.entity.override', [
              'entity_type' => $entity->getEntityTypeId(),
              'entity' => $entity->id(),
            ])->toString(),
          ],
        ] + $ipe_options;
      }
    }
    elseif ($entity instanceof PageVariant && $section_storage->getPluginId() == 'page_manager') {
      $entity_target_class = Html::getClass('layout-builder-ipe-target');
      $build['#attributes']['class'][] = $entity_target_class;
      $build['#attached']['drupalSettings']['layout_builder_ipe'] = [
        'urls' => [
          'customize' => Url::fromRoute('layout_builder_ipe.page_variant.edit', [
            'section_storage_type' => $section_storage->getStorageType(),
            'section_storage' => $section_storage->getStorageId(),
          ], [
            'query' => [
              'current_path' => Url::fromRoute('<current>')->toString(),
            ],
          ])->toString(),
        ],
        // This is tricky. We want to identify the main content element, which
        // can be different based on the used theme.
        'entity_selector' => '.' . $entity_target_class,
        'enhance_ui' => $enhance_ui,
      ];
    }

    $build['#attached']['library'][] = 'core/drupal.ajax';
    $build['#attached']['library'][] = 'layout_builder_ipe/ipe';
  }

  /**
   * Get an entity object for the current request.
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   *   The entity if found.
   */
  public function getEntity() {
    return $this->getLayoutBuilderCompatibleEntityFromRequest() ?? $this->getContentEntityFromRoute();
  }

  /**
   * Get the entity from the given section storage
   *
   * @param \Drupal\layout_builder\SectionStorageInterface $section_storage
   *   The section storage.
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   *   The entity if found.
   */
  public function getEntityFromSectionStorage(SectionStorageInterface $section_storage) {
    try {
      $context = $section_storage->getContext('entity');
      return $context->hasContextValue() ? $context->getContextValue() : NULL;
    }
    catch (ContextException $e) {
      // Fail silently.
    }
    return NULL;
  }

  /**
   * Get the current section storage if available.
   *
   * @param string $view_mode
   *   The view mode for the section storage.
   *
   * @return \Drupal\layout_builder\SectionStorageInterface|null
   *   The section storage or NULL if its context requirements are not met.
   */
  public function getSectionStorage($view_mode = 'full') {
    if ($this->routeMatch->getParameter('page_manager_page_variant')) {
      /** @var \Drupal\page_manager\PageVariantInterface $page_variant */
      $page_variant = $this->routeMatch->getParameter('page_manager_page_variant');
      $section_storage = $this->getSectionStorageForEntity($page_variant);
    }
    elseif ($this->routeMatch->getParameter('section_storage')) {
      $section_storage = $this->routeMatch->getParameter('section_storage');
    }
    elseif ($this->request->attributes->has('section_storage')) {
      $section_storage = $this->request->attributes->get('section_storage');
    }
    else {
      $entity = $this->getEntity();
      $section_storage = $entity ? $this->getSectionStorageForEntity($entity, $view_mode) : NULL;
    }
    // If we found a section storage, also check if there is a version of it in
    // the tempstore.
    return $section_storage ? $this->layoutTempstoreRepository->get($section_storage) : NULL;
  }

  /**
   * Get the section storage for an entity.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity.
   * @param string $view_mode
   *   The view mode for the section storage.
   *
   * @return \Drupal\layout_builder\SectionStorageInterface|null
   *   The section storage or NULL if its context requirements are not met.
   */
  public function getSectionStorageForEntity(EntityInterface $entity, $view_mode = 'full') {
    return $this->layoutEntityHelperGetSectionStorageForEntity($entity, $view_mode);
  }

  /**
   * Check if the current route is a possible Layout Builder IPE route.
   *
   * @return bool
   *   TRUE if Layout Builder IPE can be or is used on the current route.
   */
  public function  isLayoutBuilderIpeRoute() {
    $route_name = $this->routeMatch->getRouteName() ?? NULL;
    if ($route_name && strpos($route_name, 'layout_builder_ipe.') === 0) {
      return TRUE;
    }
    $entity = $this->getEntity();
    return $entity && $this->getSectionStorageForEntity($entity);
  }

  /**
   * Check if GIN LB is used.
   *
   * @return bool
   *   TRUE if Gin LB is installed and the current route qualifies.
   */
  public function isGinLb() {
    return $this->moduleHandler->moduleExists('gin_lb') && $this->isLayoutBuilderIpeRoute();
  }

  /**
   * Extract the main content entity from the current route.
   *
   * @return \Drupal\Core\Entity\ContentEntityInterface|null
   *   An entity object or null if none is found.
   */
  public function getContentEntityFromRoute() {
    // Entity will be found in the route parameters.
    $route = $this->routeMatch->getRouteObject();
    $parameters = $route ? $route->getOption('parameters') : NULL;
    if (!$parameters) {
      return;
    }
    foreach ($parameters as $name => $options) {
      if (isset($options['type']) && strpos($options['type'], 'entity:') === 0) {
        $entity = $this->routeMatch->getParameter($name);
        if ($entity instanceof ContentEntityInterface && $entity->hasLinkTemplate('canonical')) {
          return $entity;
        }
      }
    }
  }

  /**
   * Get an entity from the request that we can attach to.
   *
   * @return \Drupal\core\Entity\EntityInterface|null
   *   An entity object or null if none is found.
   */
  public function getLayoutBuilderCompatibleEntityFromRequest() {
    $attributes = $this->request->attributes->all();
    foreach ($attributes as $name => $attribute) {
      if (empty($attribute) || !is_object($attribute)) {
        continue;
      }
      if ($attribute instanceof EntityInterface && $this->isLayoutCompatibleEntity($attribute)) {
        return $attribute;
      }
    }
  }

  /**
   * Get the name of the active theme.
   *
   * @return string
   *   The name of currently active theme.
   */
  public function getActiveTheme() {
    return $this->themeManager->getActiveTheme()->getName();
  }

  /**
   * Get the module config.
   *
   * @param string $key
   *   The settings key.
   *
   * @return mixed
   *   The value of the requested configuration setting.
   */
  public function getConfig($key) {
    return $this->config->get($key);
  }

}