<?php

namespace Drupal\Tests\layout_builder_ipe\FunctionalJavascript;

use Drupal\FunctionalJavascriptTests\WebDriverTestBase;
use Drupal\Tests\contextual\FunctionalJavascript\ContextualLinkClickTrait;

/**
 * Tests the Layout Builder IPE UI.
 *
 * @group layout_builder_ipe
 */
class LayoutBuilderIpeUiTest extends WebDriverTestBase {

  use ContextualLinkClickTrait;

  const BUNDLE = 'bundle_with_section_field';

  /**
   * Path prefix for the field UI for the test bundle.
   *
   * @var string
   */
  const FIELD_UI_PREFIX = 'admin/structure/types/manage/' . self::BUNDLE;

  protected static $modules = [
    'layout_builder_ipe',
    'node',
    'contextual',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Create a content type.
    $this->createContentType(['type' => self::BUNDLE]);

    $this->drupalLogin($this->drupalCreateUser([
      'configure any layout',
      'create and edit custom blocks',
      'administer node display',
      'administer node fields',
      'access contextual links',
      'administer layout builder ipe',
    ]));

    // Enable layout builder.
    $this->drupalGet(static::FIELD_UI_PREFIX . '/display/default');

    $page = $this->getSession()->getPage();
    $page->find('css', '[name="layout[enabled]"]')->check();
    $page->find('css', '[name="layout[allow_custom]"]')->check();
    $page->find('css', '[name="layout[layout_builder_ipe]"]')->check();
    $page->find('css', '[value="Save"]')->click();

    $this->createNode([
      'type' => self::BUNDLE,
      'title' => "Node 1 title",
    ]);

    $this->drupalLogout();
  }

  /**
   * Tests disabling the layout page.
   */
  public function testDisableLayoutPage() {

    $this->drupalLogin($this->drupalCreateUser([
      'edit any ' . self::BUNDLE . ' content',
      'configure editable ' . self::BUNDLE . ' node layout overrides',
      'use layout builder ipe on editable ' . self::BUNDLE . ' node layout overrides',
      'create and edit custom blocks',
      'administer node display',
      'administer node fields',
      'access contextual links',
      'administer layout builder ipe',
    ]));

    // Confirm that the setting to disable the layout page is checked.
    $this->drupalGet('admin/config/user-interface/layout-builder-ipe');
    $assert_session = $this->assertSession();
    $assert_session->checkboxChecked('disable_layout_page');

    // Try to open the layout page.
    $this->drupalGet('node/1/layout');
    $assert_session->pageTextContains('You are not authorized to access this page.');

    // Confirm that the setting to disable the layout page is checked.
    $this->drupalGet('admin/config/user-interface/layout-builder-ipe');

    // Disable the setting.
    $page = $this->getSession()->getPage();
    $page->find('css', '[name="disable_layout_page"]')->uncheck();
    $page->find('css', '[value="Save configuration"]')->click();
    $assert_session->responseContains('The configuration options have been saved.');

    // Try to open the layout page.
    $this->drupalGet('node/1/layout');
    $assert_session->pageTextNotContains('You are not authorized to access this page.');
  }

  /**
   * Tests the message indicating unsaved changes.
   */
  public function testCustomizeWorkflow() {

    $this->drupalLogin($this->drupalCreateUser([
      'edit any ' . self::BUNDLE . ' content',
      'configure editable ' . self::BUNDLE . ' node layout overrides',
      'use layout builder ipe on editable ' . self::BUNDLE . ' node layout overrides',
      'create and edit custom blocks',
      'access contextual links',
    ]));

    $assert_session = $this->assertSession();
    $page = $this->getSession()->getPage();

    // Go to node view page.
    $this->drupalGet('node/1');

    // Click the customize link.
    $assert_session->elementExists('css', '#layout-builder-ipe-wrapper');
    $assert_session->elementExists('css', '.layout-builder-ipe-actions');
    $assert_session->linkExists('Customize');
    $page->clickLink('Customize');
    $assert_session->assertWaitOnAjaxRequest();
    $assert_session->waitForElementVisible('css', '#layout-builder-ipe-wrapper.edit-layout');

    // Check that the form and the three main buttons are there.
    $assert_session->elementExists('css', 'form.layout-builder-form');
    $assert_session->buttonExists('Revert to defaults');
    $assert_session->buttonExists('Discard changes');
    $assert_session->buttonExists('Save layout');

    // Save a layout without changes.
    $page->pressButton('Save layout');
    $assert_session->assertNoElementAfterWait('css', '#layout-builder-ipe-wrapper.edit-layout');
    $assert_session->pageTextContains('The layout override has been saved.');

    // Customize again to discard.
    $page->clickLink('Customize');
    $assert_session->assertWaitOnAjaxRequest();
    $assert_session->waitForElement('css', '#layout-builder-ipe-wrapper.edit-layout');
    $assert_session->buttonExists('Discard changes');
    $page->pressButton('Discard changes');
    $assert_session->responseContains('Are you sure you want to discard your layout changes?');
    $assert_session->buttonExists('Confirm');
    $page->pressButton('Confirm');
    $assert_session->assertNoElementAfterWait('css', '#layout-builder-ipe-wrapper.edit-layout');
    $assert_session->pageTextContains('The changes to the layout have been discarded.');
  }

  /**
   * Tests that IPE interface is not displayed if user has no IPE permission.
   */
  public function testNoAccessNoIpePermission() {
    $this->drupalLogin($this->drupalCreateUser([
      'edit any ' . self::BUNDLE . ' content',
      'configure editable ' . self::BUNDLE . ' node layout overrides',
      'create and edit custom blocks',
      'access contextual links',
    ]));

    // Go to node view page.
    $this->drupalGet('node/1');

    // Click the customize link.
    $assert_session = $this->assertSession();
    $assert_session->elementNotExists('css', '#layout-builder-ipe-wrapper');
    $assert_session->elementNotExists('css', '.layout-builder-ipe-actions');
    $assert_session->linkNotExists('Customize');
  }

  /**
   * Tests that IPE interface is not displayed if disabled for the view mode.
   */
  public function testNoAccessViewModeDisabled() {
    $this->drupalLogin($this->drupalCreateUser([
      'configure any layout',
      'create and edit custom blocks',
      'administer node display',
      'administer node fields',
      'access contextual links',
      'administer layout builder ipe',
    ]));

    // Enable layout builder.
    $this->drupalGet(static::FIELD_UI_PREFIX . '/display/default');

    $page = $this->getSession()->getPage();
    $page->find('css', '[name="layout[enabled]"]')->check();
    $page->find('css', '[name="layout[allow_custom]"]')->check();
    $page->find('css', '[name="layout[layout_builder_ipe]"]')->uncheck();
    $page->find('css', '[value="Save"]')->click();

    // Go to node view page.
    $this->drupalGet('node/1');

    // Click the customize link.
    $assert_session = $this->assertSession();
    $assert_session->elementNotExists('css', '#layout-builder-ipe-wrapper');
    $assert_session->elementNotExists('css', '.layout-builder-ipe-actions');
    $assert_session->linkNotExists('Customize');
  }

  /**
   * Tests that IPE interface is not displayed if user has no edit permission.
   */
  public function testNoAccessNoEditPermission() {
    $this->drupalLogin($this->drupalCreateUser([
      'configure editable ' . self::BUNDLE . ' node layout overrides',
      'use layout builder ipe on editable ' . self::BUNDLE . ' node layout overrides',
      'create and edit custom blocks',
      'access contextual links',
    ]));

    // Go to node view page.
    $this->drupalGet('node/1');

    // Click the customize link.
    $assert_session = $this->assertSession();
    $assert_session->elementNotExists('css', '#layout-builder-ipe-wrapper');
    $assert_session->elementNotExists('css', '.layout-builder-ipe-actions');
    $assert_session->linkNotExists('Customize');
  }

}
